package com.anthonycicchetti.cs5004.assignment4

import kotlin.math.pow
import kotlin.text.StringBuilder

class CleanRoom {
    data class Coefficient(val value: Int) {
        infix operator fun plus(other: Coefficient): Coefficient {
            return Coefficient(this.value + other.value)
        }

        override fun toString(): String {
            return "$value"
        }
    }

    data class Exponent(val value: Int) {
        override fun toString(): String {
            return "$value"
        }
    }


    class Polynomial() {
        constructor(inputString: String) : this() {
            inputString.split(" ").forEach {
                this.addTerm(it.substringBefore("x^").toInt(), it.substringAfter("x^").toInt())
            }
        }

        private val backingStore: MutableMap<Exponent, Coefficient> = mutableMapOf()

        fun addTerm(coefficient: Int, power: Int): Unit {
            backingStore[Exponent(power)] =
                backingStore.getOrDefault(
                    Exponent(power),
                    Coefficient(0)
                ) + Coefficient(coefficient)
        }

        fun removeTerm(power: Int) {
            if (backingStore.containsKey(Exponent(power))) {
                backingStore.remove(Exponent(power))
            }
        }

        fun getDegree(): Int {
            return backingStore.maxBy { it.key.value }?.key?.value ?: 0
        }

        fun getCoefficient(power: Int): Int {
            return backingStore.getOrDefault(
                Exponent(power),
                Coefficient(0)
            ).value
        }

        fun evaluate(xEquals: Double): Double {
            var sum = 0.0
            backingStore.forEach {
                sum += it.value.value * xEquals.pow(it.key.value)
            }
            return sum
        }

        override fun toString(): String {
            val sb = StringBuilder()

            backingStore.toSortedMap(Comparator { o1, o2 -> o2.value - o1.value })
                .forEach {
                    sb.append(it.toUsefulString())
                }

            return sb.toString()
        }
    }
}

fun Map.Entry<CleanRoom.Exponent, CleanRoom.Coefficient>.toUsefulString(): String {
    if (key.value == 0) {
        return " + ${value.value}"
    }
    return " ${(if (value.value > 0) "+" else "") + value.value}x^${key.value}"
}