package com.anthonycicchetti.cs5004.assignment4.util

import java.lang.IllegalArgumentException

interface ILinkedList<T> {
    fun addFront(entry: T): Unit
    fun addBack(entry: T): Unit
    fun addAtIndex(entry: T, index: Int): Unit
    fun getSize(): Int
    fun remove(entry: T): T
    fun getAtIndex(index: Int): T
}

open class LinkedList<T>(): ILinkedList<T> {
    private interface ILinkedListNode<T> {
        fun count(): Int
        fun addFront(entry: T): ILinkedListNode<T>
        fun addBack(entry: T): ILinkedListNode<T>
        fun addAtIndex(entry: T, index: Int): ILinkedListNode<T>
        fun remove(entry: T): ILinkedListNode<T>
        fun getAtIndex(index: Int): T
    }

    private class EmptyLinkedListNode<T>(): ILinkedListNode<T> {
        override fun count(): Int = 0

        override fun addFront(entry: T): ILinkedListNode<T> = LinkedListNode(entry, this)

        override fun addBack(entry: T): ILinkedListNode<T> = addFront(entry)

        override fun addAtIndex(entry: T, index: Int): ILinkedListNode<T> = if (index == 0) addFront(entry) else throw IllegalArgumentException("Index must be 0")

        override fun remove(entry: T): ILinkedListNode<T> = this

        override fun getAtIndex(index: Int): T = throw IllegalArgumentException("Wrong index")

    }
    
    private class LinkedListNode<T>(val data: T, var rest: ILinkedListNode<T>): ILinkedListNode<T> {
        override fun count(): Int = 1 + rest.count()

        override fun addFront(entry: T) = LinkedListNode(entry, this)

        override fun addBack(entry: T): ILinkedListNode<T> {
            this.rest = this.rest.addBack(entry)
            return this
        }

        override fun addAtIndex(entry: T, index: Int): ILinkedListNode<T> {
            return if (index == 0) {
                addFront(entry)
            } else {
                this.rest = this.rest.addAtIndex(entry, index - 1)
                this
            }
        }

        override fun remove(entry: T): ILinkedListNode<T> {
            return if (this.data == entry) {
                this.rest
            } else {
                this.rest = this.rest.remove(entry)
                this
            }
        }

        override fun getAtIndex(index: Int): T {
            return if (index == 0) {
                this.data
            } else {
                this.rest.getAtIndex(index - 1)
            }
        }
    }
    private var head: ILinkedListNode<T> = EmptyLinkedListNode<T>()

    override fun addFront(entry: T) {
        head = head.addFront(entry)
    }

    override fun addBack(entry: T) {
        head = head.addBack(entry)
    }

    override fun addAtIndex(entry: T, index: Int) {
        head = head.addAtIndex(entry, index)
    }

    override fun getSize(): Int {
        return head.count()
    }

    override fun remove(entry: T): T {
        head = head.remove(entry)
        return entry
    }

    override fun getAtIndex(index: Int): T {
        if ((index >= 0) && (index < this.getSize())) {
            return head.getAtIndex(index)
        } else throw IllegalArgumentException("Invalid Index")
    }

}