package com.anthonycicchetti.cs5004.assignment5

import com.anthonycicchetti.cs5004.assignment5.model.Tile
import java.lang.IllegalArgumentException
import kotlin.math.absoluteValue

class Board() : MarbleSolitaireModel {
    val backingState: MutableList<Tile>

    private fun invalidBoardTile(row: Int, column: Int): Boolean {
        return ((row <= 1 || row >= 5)
                && (column <= 1 || column >= 5))
    }

    constructor(emptyRow: Int, emptyColumn: Int) : this() {
        if (!invalidBoardTile(emptyRow, emptyColumn)) {
            this.backingState.remove(Tile(3, 3, false))
            this.backingState.add(Tile(3, 3, true))
            this.backingState.remove(Tile(emptyRow, emptyColumn, true))
            this.backingState.add(Tile(emptyRow, emptyColumn, false))
        } else {
            throw IllegalArgumentException("Invalid empty cell position ($emptyRow, $emptyColumn)")
        }
    }

    init {
        val eventualBackingState = mutableListOf<Tile>()
        for (column in 0..6) {
            for (row in 0..6) {
                if (invalidBoardTile(row, column)) {
                    eventualBackingState.add(Tile(row, column, null))
                } else {
                    eventualBackingState.add(Tile(row, column, true))
                }
            }
        }
        if (eventualBackingState.contains(Tile(3, 3, true))) {
            eventualBackingState.remove(Tile(3, 3, true))
            eventualBackingState.add(Tile(3, 3, false))
        }
        backingState = eventualBackingState.sorted().toMutableList()
    }

    private fun tileIsOccupied(row: Int, column: Int): Boolean {
        return backingState.contains(Tile(row, column, true))
    }

    private fun getTileAt(row: Int, column: Int): Tile {
        return backingState.filter { it.row == row && it.column == column }[0]
    }

    private fun validMove(fromRow: Int, fromCol: Int, toRow: Int, toCol: Int): Boolean {
        if ((fromRow - toRow).absoluteValue != 2 && (fromCol - toCol == 0)
            || (fromCol - toCol).absoluteValue != 2 && (fromRow - toRow == 0)) {
            return false
        }
        if (invalidBoardTile(fromRow, fromCol) || invalidBoardTile(toRow, toCol)) {
            return false
        }
        val avgRow = (fromRow + toRow) / 2
        val avgCol = (fromCol + toCol) / 2
        if (tileIsOccupied(avgRow, avgCol)
            && !tileIsOccupied(toRow, toCol)
            && tileIsOccupied(fromRow, fromCol)){
            return true
        }
        return false
    }

    override fun move(fromRow: Int, fromCol: Int, toRow: Int, toCol: Int) {
        if (validMove(fromRow, fromCol, toRow, toCol)) {
            val avgRow = (fromRow + toRow) / 2
            val avgCol = (fromCol + toCol) / 2

            backingState.remove(Tile(fromRow, fromCol, true))
            backingState.add(Tile(fromRow, fromCol, false))
            backingState.remove(Tile(avgRow, avgCol, true))
            backingState.add(Tile(avgRow, avgCol, false))
            backingState.remove(Tile(toRow, toCol, false))
            backingState.add(Tile(toRow, toCol, true))
        }

        backingState.sort()
    }

    override fun isGameOver(): Boolean {
        if (getScore() == 1) {
            return true
        } else {
            for (tile in backingState) {
                val localRow = tile.row
                val localCol = tile.column
                val plusRow = localRow + 2
                val plusCol = tile.column + 2
                val minusRow = localRow - 2
                val minusCol = tile.column - 2
                if (validMove(localRow, localCol, plusRow, localCol)
                    || validMove(localRow, localCol, localRow, plusCol)
                    || validMove(localRow, localCol, minusRow, localCol)
                    || validMove(localRow, localCol, localRow, minusCol)) {
                    return false
                }
            }
        }
        return true
    }

    override fun getGameState(): String {
        val sb = StringBuilder()
        var rowBuilder = StringBuilder()

        for (row in 0..6) {
            backingState.filter { it.row == row }.sorted().forEach {
                rowBuilder.append(
                    when (it.occupied) {
                        true -> "O"
                        false -> "X"
                        null -> " "
                    }
                ).append(" ")
            }
            sb.append(rowBuilder.toString()).append("\n")
            rowBuilder = StringBuilder()
        }
        return sb.toString().trimEnd('\n')
    }

    override fun getScore(): Int = backingState.count { it.occupied ?: false }

}