package com.anthonycicchetti.cs5004.assignment5

import com.anthonycicchetti.cs5004.assignment5.Board
import com.anthonycicchetti.cs5004.assignment5.model.Tile
import org.junit.jupiter.api.Test

import org.junit.jupiter.api.Assertions.*

internal class BoardTest {
    @Test
    fun `A Valid Board is Created By Default`() {
        val board = Board()
        val expected =
            """    O O O     
    O O O     
O O O O O O O 
O O O X O O O 
O O O O O O O 
    O O O     
    O O O     """

        assertEquals(expected, board.getGameState())
    }

    @Test
    fun `A Valid Board is Created With a Different Empty Hole`() {
        val board = Board(2, 3)

        assertFalse(board.backingState.contains(Tile(2, 3, true)))
    }

    @Test
    fun `A Different-Holed Board is Printed Correctly`() {
        val board = Board(2,3)
        val expected =
"""    O O O     
    O O O     
O O O X O O O 
O O O O O O O 
O O O O O O O 
    O O O     
    O O O     """

        assertEquals(expected, board.getGameState())
    }

    @Test
    fun `Board is printed correctly after a move`() {
        val board = Board()
        val movedBoard =
"""    O O O     
    O O O     
O O O O O O O 
O X X O O O O 
O O O O O O O 
    O O O     
    O O O     """

        board.move(3,1,3,3)
        assertEquals(movedBoard, board.getGameState())
    }

    @Test
    fun `Score is counted correctly on new board`() {
        val board = Board()

        assertEquals(32, board.getScore())
    }

    @Test
    fun `Score is counted correct on new non-default board`() {
        val board = Board(1,4)

        assertEquals(32, board.getScore())
    }

    @Test
    fun `Score is counted correctly on board with missing marbles`() {
        val board = Board()

        board.move(3, 1, 3, 3)
        assertEquals(31, board.getScore())
    }

    @Test
    fun `Game is not over when game is started`() {
        val board = Board()

        assertFalse(board.isGameOver())
    }

    @Test
    fun `Game is not over when game is started with a nonstandard board`() {
        val board = Board(2,3)

        assertFalse(board.isGameOver())
    }

    @Test
    fun `Game is not over after one move`() {
        val board = Board()

        board.move(3, 1, 3, 3)
        assertFalse(board.isGameOver())
    }


}
